import time

# Import the preset TestFixture.
from .Frames_capture_fixture import FixtureFramesCapture, ChannelFactory

from xtr import logging, Severity
logger = logging.getLogger(__name__)


# Create the test case class, that inherits from the preset TestFixture.
class TestCaptureCanFdFrames(FixtureFramesCapture):

    # This method will be executed just before
    # the test case method execution.
    def setUp(self):
        logger.info("Setup test case")

    # This method will be executed just after
    # the test case method execution.
    def tearDown(self):
        logger.info("Tear down test case")

    # Create the test case method that holds the main test steps.
    def test_capture_can_fd_frames(self):

        # Define frame properties.
        # The values provided below are arbitrary examples.
        # They do not belong to any actual in vehicle communication.
        channel_name = 'B1_CAN_FD'
        frame_id = 0x17
        payload = [0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7,
                   0x8, 0x9, 0x10, 0x11, 0x12]

        can_fd_ch = ChannelFactory().get_can_channel(channel_name, True)

        # Retrieves the desired can frame object associated with the the frame_id.
        can_fd_fr = can_fd_ch.get_frame(frame_id)

        # Start listening on the can fd channel.
        can_fd_ch.listener.start_listening()

        # Log any useful message.
        logger.info("Transmit a can fd frame")

        # Transmit the frame.
        can_fd_fr.transmitter.transmit_frame(payload)

        # Pause the program for 3 seconds.
        time.sleep(3)

        # Stop listening on the can fd channel
        can_fd_ch.listener.stop_listening()

        q = can_fd_ch.listener.get_queue()

        # Confirm that the queue is not Null object,
        # meaning there is no error in creating the queue.
        self.assertTrue(q is not None, Severity.BLOCKER,
                        "Error when listening to the can fd bus")

        # Confirm that queue is not empty meaning messages are received.
        self.assertTrue(q.qsize() >= 1, Severity.BLOCKER,
                        "Can fd frame is not received")

        event = q.queue[0]
        logger.info(repr(event))

        logger.info(f"{event.channel_name}")
        logger.info(f"{event.frame_id}")